/**
 * @license Copyright 2021 The Lighthouse Authors. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.
 */

import {jest} from '@jest/globals';
import {FunctionComponent} from 'preact';
import {act, render} from '@testing-library/preact';

import {Topbar} from '../src/topbar';
import {FlowResultContext} from '../src/util';
import {ReportRendererContext} from '../src/wrappers/report-renderer';
import {I18nProvider} from '../src/i18n/i18n';

const flowResult = {
  name: 'User flow',
  steps: [{lhr: {
    fetchTime: '2021-09-14T22:24:22.462Z',
    configSettings: {locale: 'en-US'},
    i18n: {rendererFormattedStrings: {}},
  }}],
} as any;

let wrapper: FunctionComponent;
let mockSaveFile = jest.fn();

beforeEach(() => {
  mockSaveFile = jest.fn();
  const reportRendererValue: any = {
    dom: {
      saveFile: mockSaveFile,
    },
  };
  wrapper = ({children}) => (
    <FlowResultContext.Provider value={flowResult}>
      <ReportRendererContext.Provider value={reportRendererValue}>
        <I18nProvider>
          {children}
        </I18nProvider>
      </ReportRendererContext.Provider>
    </FlowResultContext.Provider>
  );
});

it('save button opens save dialog for HTML file', async () => {
  const root = render(<Topbar onMenuClick={() => {}}/>, {wrapper});

  const saveButton = root.getByText('Save');
  saveButton.click();

  expect(mockSaveFile).toHaveBeenCalledWith(
    expect.any(Blob),
    'User-flow_2021-09-14_22-24-22'
  );
});

it('toggles help dialog', async () => {
  const root = render(<Topbar onMenuClick={() => {}}/>, {wrapper});

  expect(root.queryByText(/Use Navigation reports to/)).toBeFalsy();
  const helpButton = root.getByText('Understanding Flows');

  await act(() => helpButton.click());
  expect(root.getByText(/Use Navigation reports to/)).toBeTruthy();

  await act(() => helpButton.click());
  expect(root.queryByText(/Use Navigation reports to/)).toBeFalsy();
});
